using System;
using MbUnit.Framework;
using Rhino.Mocks.Constraints;

namespace Rhino.Mocks.Demo
{
	[TestFixture]
	public class RhinoMocks101Fixture
	{
		private MockRepository mockRepository;
		private ISmsSender smsSender;
		private IUserRepository userRepository;
		private User user;
		private LoginController loginController;

		[SetUp]
		public void TestInitialize()
		{
			mockRepository = new MockRepository();
			smsSender = mockRepository.CreateMock<ISmsSender>();
			userRepository = mockRepository.CreateMock<IUserRepository>();
			user = CreateUser();

			loginController = new LoginController(smsSender, this.userRepository);
		}

		[TearDown]
		public void TestCleanup()
		{
			mockRepository.ReplayAll();
			mockRepository.VerifyAll();
		}

		[Test]
		public void WillSendSMSWhenUserRequestNewPassword()
		{
			Expect.Call(this.userRepository.GetById("sally")).Return(this.user);
			smsSender.SendSms(null, null);
			LastCall.Constraints(Is.Equal(this.user.Phone), Text.StartsWith("New passord:"));

			mockRepository.ReplayAll();

			loginController.SendNewPassword(this.user.Name);

			Assert.AreNotEqual("old", this.user.Password, "expected the password to change");
		}

		private static User CreateUser()
		{
			User user = new User("sally", "1234-4321");
			user.Password = "old";
			return user;
		}

		[Test]
		public void CalledWithInvalidUserId()
		{
			Expect.Call(this.userRepository.GetById(null))
				.Constraints(!Is.Equal("sally"))
				.Return(null)
				.Message("Should be called with an invalid user id, and return null as result");

			smsSender.SendSms(null, null);
			LastCall.IgnoreArguments().Repeat.Never()
				.Message("Should not be called for invalid user id");

			mockRepository.ReplayAll();

			loginController.SendNewPassword("not sally");
		}

		[Test]
		public void CanRecoverFromFailuresInSendingSms_WillNotChangePassword()
		{
			Expect.Call(this.userRepository.GetById("sally"))
					.Return(this.user);
			smsSender.SendSms(null, null);
			LastCall.Constraints(Is.Equal(this.user.Phone), Text.StartsWith("New passord:"))
				.Throw(new InvalidOperationException("SMS Server down"));

			user.Password = "old";

			mockRepository.ReplayAll();

			try
			{
				loginController.SendNewPassword(user.Name);
			}
			catch
			{
				Assert.Fail("Even though the sms sender threw an exception, it should not escape the login controller");
			}

			Assert.AreEqual("old", user.Password );
		}

		public delegate void SendSmsDelegate(string destination, string message);

		[Test]
		public void WillIssueWarningAbout_SLA_Violations_IfCallMoreThanExpectedDuration()
		{
			Expect.Call(this.userRepository.GetById("sally")).Return(this.user);
			smsSender.SendSms(null, null);
			LastCall.Constraints(Is.Equal(this.user.Phone), Text.StartsWith("New passord:"));

			mockRepository.ReplayAll();
			loginController.MaxAllowedDuration = TimeSpan.FromSeconds(-3);
			loginController.SendNewPassword(this.user.Name);

			Assert.AreEqual(1, loginController.SlaVilations.Count, "Should have added an SLA violation" );
		}
	}
}